Package: triebeard
Type: Package
Title: 'Radix' Trees in 'Rcpp'
Version: 0.4.1
Author: Os Keyes [aut, cre], Drew Schmidt [aut], Yuuki Takano [cph]
Maintainer: Os Keyes <ironholds@gmail.com>
Description: 'Radix trees', or 'tries', are key-value data structures optimised for efficient lookups, similar in purpose
             to hash tables. 'triebeard' provides an implementation of 'radix trees' for use in R programming and in
             developing packages with 'Rcpp'.
License: MIT + file LICENSE
LinkingTo: Rcpp
Encoding: UTF-8
Imports: Rcpp
RoxygenNote: 7.1.2
Suggests: knitr, rmarkdown, testthat
VignetteBuilder: knitr
URL: https://github.com/Ironholds/triebeard/
BugReports: https://github.com/Ironholds/triebeard/issues
Date: 2023-03-04
NeedsCompilation: yes
Packaged: 2023-03-04 21:40:31 UTC; ironholds
Repository: CRAN
Date/Publication: 2023-03-04 23:30:07 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 15:55:08 UTC; unix
Archs: triebeard.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,trie)
S3method(as.list,trie)
S3method(dim,trie)
S3method(get_keys,integer_trie)
S3method(get_keys,logical_trie)
S3method(get_keys,numeric_trie)
S3method(get_keys,string_trie)
S3method(get_values,integer_trie)
S3method(get_values,logical_trie)
S3method(get_values,numeric_trie)
S3method(get_values,string_trie)
S3method(greedy_match,integer_trie)
S3method(greedy_match,logical_trie)
S3method(greedy_match,numeric_trie)
S3method(greedy_match,string_trie)
S3method(length,integer_trie)
S3method(length,logical_trie)
S3method(length,numeric_trie)
S3method(length,string_trie)
S3method(longest_match,integer_trie)
S3method(longest_match,logical_trie)
S3method(longest_match,numeric_trie)
S3method(longest_match,string_trie)
S3method(prefix_match,integer_trie)
S3method(prefix_match,logical_trie)
S3method(prefix_match,numeric_trie)
S3method(prefix_match,string_trie)
S3method(print,trie)
S3method(str,trie)
S3method(trie_add,integer_trie)
S3method(trie_add,logical_trie)
S3method(trie_add,numeric_trie)
S3method(trie_add,string_trie)
S3method(trie_remove,integer_trie)
S3method(trie_remove,logical_trie)
S3method(trie_remove,numeric_trie)
S3method(trie_remove,string_trie)
export(get_keys)
export(get_values)
export(greedy_match)
export(longest_match)
export(prefix_match)
export(trie)
export(trie_add)
export(trie_remove)
importFrom(Rcpp,sourceCpp)
useDynLib(triebeard, .registration = TRUE)

## triebeard

Fast key-value matching in R and Rcpp

__Author:__ Os Keyes, Drew Schmidt, Yuuki Takano<br/>
__License:__ [MIT](https://opensource.org/license/mit/)<br/>
__Status:__ Stable

[![Travis-CI Build Status](https://travis-ci.org/Ironholds/triebeard.svg?branch=master)](https://travis-ci.org/Ironholds/triebeard) ![downloads](http://cranlogs.r-pkg.org/badges/grand-total/triebeard)

### Description

Tries, or [radix trees](https://en.wikipedia.org/wiki/Radix_tree), are key-value data structures optimised for very, very fast matching of the keys against user-provided data (and then the return of the associated values!)

This is pretty useful in data cleaning and value extraction, and tries let you do it *really* efficiently. `triebeard` contains
an implementation that can be used both when writing R, and when writing Rcpp (and imported and linked against, to boot). For more information see:

1. The [vignette on Rcpp usage](https://CRAN.R-project.org/package=triebeard/vignettes/rcpp_radix.html);
2. The [vignette on R usage](https://CRAN.R-project.org/package=triebeard/vignettes/r_radix.html).

Please note that this project is released with a [Contributor Code of Conduct](https://github.com/Ironholds/triebeard/blob/master/CONDUCT.md).
By participating in this project you agree to abide by its terms.

### Installation

The stable, CRAN-ready version can be retrieved with:

    install.packages("triebeard")

The latest version can be obtained via:

    devtools::install_github("ironholds/triebeard")

### Dependencies
* R.
* [Rcpp](https://cran.r-project.org/package=Rcpp)

## //[[Rcpp::depends(triebeard)]]

## #include <radix.h>


## radix_tree<type1, type2> radix;


## radix_tree<std::string, std::string> radix;

## radix["turnin"] = "entry the first";

## radix["turin"] = "entry the second";

## 
## radix_tree<std::string, std::string>::iterator it;

## 
## it = radix.longest_match("turing");

## 
## if(it = radix.end()){

##   printf("No match was found :(");

## } else {

##   std::string result = "Key of longest match: " + it->first + " , value of longest match: " + it->second;

## }


## radix_tree<std::string, std::string> radix;

## radix["turnin"] = "entry the first";

## radix["turin"] = "entry the second";

## 
## std::vector<radix_tree<std::string, std::string>::iterator> vec;

## std::vector<radix_tree<std::string, std::string>::iterator>::iterator it;

## 
## it = radix.prefix_match("tur");

## 
## if(it == vec.end()){

##   printf("No match was found :(");

## } else {

##   for (it = vec.begin(); it != vec.end(); ++it) {

##     std::string result = "Key of a prefix match: " + it->first + " , value of a prefix match: " + it->second;

##   }

## }

#'@title Trie Getters
#'@description "Getters" for the data stored in a trie object. \code{get_keys}
#' gets the keys, \code{get_values} gets the values.
#'
#'@param trie A trie object, created with \code{\link{trie}}.
#'
#'@return An atomic vector of keys or values stored in the trie.
#'
#'@name getters
#'@rdname getters
NULL

#'@rdname getters
#'@export
get_keys <- function(trie){
  stopifnot("trie" %in% class(trie))
  UseMethod("get_keys", trie)
}

#'@rdname getters
#'@export
get_values <- function(trie){
  stopifnot("trie" %in% class(trie))
  UseMethod("get_values", trie)
}

#'@export
get_keys.string_trie <- function(trie){
  return(get_keys_string(trie))
}

#'@export
get_keys.integer_trie <- function(trie){
  return(get_keys_integer(trie))
}

#'@export
get_keys.numeric_trie <- function(trie){
  return(get_keys_numeric(trie))
}

#'@export
get_keys.logical_trie <- function(trie){
  return(get_keys_logical(trie))
}

#'@export
get_values.string_trie <- function(trie){
  return(get_values_string(trie))
}

#'@export
get_values.integer_trie <- function(trie){
  return(get_values_integer(trie))
}

#'@export
get_values.numeric_trie <- function(trie){
  return(get_values_numeric(trie))
}

#'@export
get_values.logical_trie <- function(trie){
  return(get_values_logical(trie))
}

context("Test key and value retrieval")

testthat::test_that("String keys and values can be retrieved", {
  string_trie <- trie(LETTERS, LETTERS)
  testthat::expect_equal(LETTERS, get_values(string_trie))
  testthat::expect_equal(LETTERS, get_keys(string_trie))
})

testthat::test_that("Integer keys and values can be retrieved", {
  int_trie <- trie(LETTERS, 1:length(LETTERS))
  testthat::expect_equal(1:length(LETTERS), get_values(int_trie))
  testthat::expect_equal(LETTERS, get_keys(int_trie))
})

testthat::test_that("Numeric keys and values can be retrieved", {
  vals <- as.double(1:length(LETTERS))
  double_trie <- trie(LETTERS, vals)

  testthat::expect_equal(vals, get_values(double_trie))
  testthat::expect_equal(LETTERS, get_keys(double_trie))
})

testthat::test_that("Boolean keys and values can be retrieved", {
  vals <- as.logical(rep(c(0,1), (length(LETTERS)/2)))
  bool_trie <- trie(LETTERS, vals)

  testthat::expect_equal(vals, get_values(bool_trie))
  testthat::expect_equal(LETTERS, get_keys(bool_trie))
})

#' @title Radix trees in Rcpp
#' @name triebeard
#' @description This package provides access to Radix tree (or "trie") structures in Rcpp. At
#' a later date it will hopefully provide them in R, too.
#'
#' @docType package
#' @aliases triebeard triebeard-package
#' @useDynLib triebeard
#' @importFrom Rcpp sourceCpp
NULL

#'@title Find the longest match in a trie
#'@description \code{longest_match} accepts a trie and a character vector
#'and returns the value associated with whichever key had the \emph{longest match}
#'to each entry in the character vector. A trie of "binary" and "bind", for example,
#'with an entry-to-compare of "binder", will match to "bind".
#'
#'@param trie a trie object, created with \code{\link{trie}}
#'
#'@param to_match a character vector containing the strings to match against the
#'trie's keys.
#'
#'@examples
#'trie <- trie(keys = c("afford", "affair", "available", "binary", "bind", "blind"),
#'                    values = c("afford", "affair", "available", "binary", "bind", "blind"))
#'longest_match(trie, "binder")
#'
#'@seealso \code{\link{prefix_match}} and \code{\link{greedy_match}}
#'for prefix and greedy matching, respectively.
#'
#'@export
longest_match <- function(trie, to_match){
  stopifnot("trie" %in% class(trie))
  UseMethod("longest_match", trie)
}

#'@export
longest_match.string_trie <- function(trie, to_match){
  return(longest_string(trie, to_match))
}

#'@export
longest_match.integer_trie <- function(trie, to_match){
  return(longest_integer(trie, to_match))
}

#'@export
longest_match.numeric_trie <- function(trie, to_match){
  return(longest_numeric(trie, to_match))
}

#'@export
longest_match.logical_trie <- function(trie, to_match){
  return(longest_logical(trie, to_match))
}

#'@title Find the prefix matches in a trie
#'@description \code{prefix_match} accepts a trie and a character vector
#'and returns the values associated with any key that has a particular
#'character vector entry as a prefix (see the examples).
#'
#'@param trie a trie object, created with \code{\link{trie}}
#'
#'@param to_match a character vector containing the strings to check against the
#'trie's keys.
#'
#'@return a list, the length of \code{to_match}, with each entry containing any trie values
#'where the \code{to_match} element was a prefix of the associated key. In the case that
#'nothing was found, the entry will contain \code{NA}.
#'
#'@examples
#'trie <- trie(keys = c("afford", "affair", "available", "binary", "bind", "blind"),
#'                    values = c("afford", "affair", "available", "binary", "bind", "blind"))
#'prefix_match(trie, "aff")
#'
#'@seealso \code{\link{longest_match}} and \code{\link{greedy_match}}
#'for longest and greedy matching, respectively.
#'
#'@export
prefix_match <- function(trie, to_match){
  stopifnot("trie" %in% class(trie))
  UseMethod("prefix_match", trie)
}

#'@export
prefix_match.numeric_trie <- function(trie, to_match){
  return(prefix_numeric(trie, to_match))
}

#'@export
prefix_match.integer_trie <- function(trie, to_match){
  return(prefix_integer(trie, to_match))
}

#'@export
prefix_match.string_trie <- function(trie, to_match){
  return(prefix_string(trie, to_match))
}

#'@export
prefix_match.logical_trie <- function(trie, to_match){
  return(prefix_logical(trie, to_match))
}

#'@title Greedily match against a tree
#'@description \code{greedy_match} accepts a trie and a character vector
#'and returns the values associated with any key that is "greedily" (read: fuzzily)
#'matched against one of the character vector entries.
#'
#'@param trie a trie object, created with \code{\link{trie}}
#'
#'@param to_match a character vector containing the strings to check against the
#'trie's keys.
#'
#'@return a list, the length of \code{to_match}, with each entry containing any trie values
#'where the \code{to_match} element greedily matches the associated key. In the case that
#'nothing was found, the entry will contain \code{NA}.
#'
#'@examples
#'trie <- trie(keys = c("afford", "affair", "available", "binary", "bind", "blind"),
#'                    values = c("afford", "affair", "available", "binary", "bind", "blind"))
#'greedy_match(trie, c("avoid", "bring", "attack"))
#'
#'@seealso \code{\link{longest_match}} and \code{\link{prefix_match}}
#'for longest and prefix matching, respectively.
#'
#'@export
greedy_match <- function(trie, to_match){
  stopifnot("trie" %in% class(trie))
  UseMethod("greedy_match", trie)
}

#'@export
greedy_match.numeric_trie <- function(trie, to_match){
  return(greedy_numeric(trie, to_match))
}


#'@export
greedy_match.integer_trie <- function(trie, to_match){
  return(greedy_integer(trie, to_match))
}

#'@export
greedy_match.string_trie <- function(trie, to_match){
  return(greedy_string(trie, to_match))
}

#'@export
greedy_match.logical_trie <- function(trie, to_match){
  return(greedy_logical(trie, to_match))
}

testthat::context("Test that prefix-matching works")

testthat::test_that("prefix matching works for string tries", {
  trie <- trie(keys = c("afford", "affair", "available", "binary", "bind", "blind"),
                      values = c("afford", "affair", "available", "binary", "bind", "blind"))
  output <- prefix_match(trie, "bin")
  testthat::expect_equal(length(output), 1)
  testthat::expect_true(is.list(output))
  testthat::expect_true(all(output[[1]] == c("binary", "bind")))
})

testthat::test_that("prefix matching works for integer tries", {
  trie <- trie(keys = c("afford", "affair", "available", "binary", "bind", "blind"),
                      values = c(1, 2, 3, 4, 5, 6))
  output <- prefix_match(trie, "bin")
  testthat::expect_equal(length(output), 1)
  testthat::expect_true(is.list(output))
  testthat::expect_true(all(output[[1]] == c(4, 5)))
})

testthat::test_that("prefix matching works for numeric tries", {
  trie <- trie(keys = c("afford", "affair", "available", "binary", "bind", "blind"),
                      values = as.numeric(c(1, 2, 3, 4, 5, 6)))
  output <- prefix_match(trie, "bin")
  testthat::expect_equal(length(output), 1)
  testthat::expect_true(is.list(output))
  testthat::expect_true(all(output[[1]] == c(4, 5)))
})

testthat::test_that("prefix matching works for logical tries", {
  trie <- trie(keys = c("afford", "affair", "available", "binary", "bind", "blind"),
                      values = c(FALSE, FALSE, TRUE, FALSE, TRUE, TRUE))
  output <- prefix_match(trie, "bin")
  testthat::expect_equal(length(output), 1)
  testthat::expect_true(is.list(output))
  testthat::expect_true(all(output[[1]] == c(FALSE, TRUE)))
})

testthat::test_that("prefix matching objects to non-trie objects", {
  expect_error(prefix_match("foo", "bar"))
})

testthat::test_that("prefix matching produces NAs with impossibilities", {
  trie <- trie(keys = c("afford", "affair", "available", "binary", "bind", "blind"),
                      values = c(FALSE, FALSE, TRUE, FALSE, TRUE, TRUE))
  output <- prefix_match(trie, "bingo")
  testthat::expect_equal(length(output), 1)
  testthat::expect_true(is.list(output))
  testthat::expect_true(is.na(output[[1]]))
})

testthat::context("Test that longest-matching works")

testthat::test_that("Longest matching works for string tries", {
  trie <- trie(keys = c("afford", "affair", "available", "binary", "bind", "blind"),
                      values = c("afford", "affair", "available", "binary", "bind", "blind"))
  testthat::expect_equal(longest_match(trie, "binder"), "bind")
})

testthat::test_that("Longest matching works for integer tries", {
  trie <- trie(keys = c("afford", "affair", "available", "binary", "bind", "blind"),
                      values = c(1, 2, 3, 4, 5, 6))
  testthat::expect_equal(longest_match(trie, "binder"), 5)
})

testthat::test_that("Longest matching works for numeric tries", {
  trie <- trie(keys = c("afford", "affair", "available", "binary", "bind", "blind"),
                      values = as.numeric(c(1, 2, 3, 4, 5, 6)))
  testthat::expect_equal(longest_match(trie, "binder"), 5.0)
})

testthat::test_that("Longest matching works for logical tries", {
  trie <- trie(keys = c("afford", "affair", "available", "binary", "bind", "blind"),
                      values = c(FALSE, FALSE, TRUE, FALSE, TRUE, TRUE))
  testthat::expect_true(longest_match(trie, "binder"))
})

testthat::test_that("Longest matching objects to non-trie objects", {
  expect_error(longest_match("foo", "bar"))
})

testthat::context("Test that greedy-matching works")

testthat::test_that("greedy matching works for string tries", {
  trie <- trie(keys = c("afford", "affair", "available", "binary", "bind", "blind"),
                      values = c("afford", "affair", "available", "binary", "bind", "blind"))
  output <- greedy_match(trie, "avoid")
  testthat::expect_equal(length(output), 1)
  testthat::expect_true(is.list(output))
  testthat::expect_true(all(output[[1]] == "available"))
})

testthat::test_that("greedy matching works for integer tries", {
  trie <- trie(keys = c("afford", "affair", "available", "binary", "bind", "blind"),
                      values = c(1, 2, 3, 4, 5, 6))
  output <- greedy_match(trie, "avoid")
  testthat::expect_equal(length(output), 1)
  testthat::expect_true(is.list(output))
  testthat::expect_true(all(output[[1]] == 3))
})

testthat::test_that("greedy matching works for numeric tries", {
  trie <- trie(keys = c("afford", "affair", "available", "binary", "bind", "blind"),
                      values = as.numeric(c(1, 2, 3, 4, 5, 6)))
  output <- greedy_match(trie, "avoid")
  testthat::expect_equal(length(output), 1)
  testthat::expect_true(is.list(output))
  testthat::expect_true(all(output[[1]] == 3))
})

testthat::test_that("greedy matching works for logical tries", {
  trie <- trie(keys = c("afford", "affair", "available", "binary", "bind", "blind"),
                      values = c(FALSE, FALSE, TRUE, FALSE, TRUE, TRUE))
  output <- greedy_match(trie, "avoid")
  testthat::expect_equal(length(output), 1)
  testthat::expect_true(is.list(output))
  testthat::expect_true(output[[1]])
})

testthat::test_that("greedy matching objects to non-trie objects", {
  expect_error(greedy_match("foo", "bar"))
})

#'@export
as.list.trie <- function(x, ...){
  return(list(keys = get_keys(x), values = get_values(x)))
}

#'@export
as.data.frame.trie <- function(x, row.names = NULL, optional = FALSE,
                               stringsAsFactors = FALSE, ...){

  output <- data.frame(keys = get_keys(x),
                       values = get_values(x),
                       stringsAsFactors = stringsAsFactors,
                       ...)
  if(!is.null(row.names)){
    rownames(output) <- row.names
  }
  return(output)
}

#'@title Create a Trie
#'@description \code{create_trie} creates a trie (a key-value store optimised
#'for matching) out of a provided character vector of keys, and a numeric,
#'character, logical or integer vector of values (both the same length).
#'
#'@param keys a character vector containing the keys for the trie.
#'
#'@param values an atomic vector of any type, containing the values to pair with
#'\code{keys}. Must be the same length as \code{keys}.
#'
#'@return a `trie` object.
#'
#'@seealso \code{\link{trie_add}} and \code{\link{trie_remove}} for adding to and removing
#'from tries after their creation, and \code{\link{longest_match}} and other match functions
#'for matching values against the keys of a created trie.
#'
#'@examples
#'# An integer trie
#'int_trie <- trie(keys = "foo", values = 1)
#'
#'# A string trie
#'str_trie <- trie(keys = "foo", values = "bar")
#'
#'@export
trie <- function(keys, values){
  stopifnot(length(keys) == length(values))
  stopifnot(is.character(keys))
  output <- NULL
  output_classes <- c("trie", NA)

  switch(class(values)[1],
         "character" = {
           output <- radix_create_string(keys, values)
           output_classes[2] <- "string_trie"
         },
         "integer" = {
           output <- radix_create_integer(keys, values)
           output_classes[2] <- "integer_trie"
         },
         "numeric" = {
           output <- radix_create_numeric(keys, values)
           output_classes[2] <- "numeric_trie"
         },
         "logical" = {
           output <- radix_create_logical(keys, values)
           output_classes[2] <- "logical_trie"
         },
         stop("'values' must be a numeric, integer, character or logical vector"))

  class(output) <- c(class(output), output_classes)
  return(output)
}

context("Test trie creation")

testthat::test_that("String tries can be created and safely avoid collection", {
  string_trie <- trie(LETTERS, LETTERS)
  testthat::expect_true(any(class(string_trie) == "string_trie"))
  testthat::expect_true(any(class(string_trie) == "trie"))
})

testthat::test_that("Integer tries can be created", {
  int_trie <- trie(LETTERS, 1:length(LETTERS))

  testthat::expect_true(any(class(int_trie) == "integer_trie"))
  testthat::expect_true(any(class(int_trie) == "trie"))
})

testthat::test_that("Double tries can be created", {
  vals <- as.double(1:length(LETTERS))
  double_trie <- trie(LETTERS, vals)

  testthat::expect_true(any(class(double_trie) == "numeric_trie"))
  testthat::expect_true(any(class(double_trie) == "trie"))
})

testthat::test_that("Logical tries can be created", {
  vals <- as.logical(rep(c(0,1), (length(LETTERS)/2)))
  bool_trie <- trie(LETTERS, vals)

  testthat::expect_true(any(class(bool_trie) == "logical_trie"))
  testthat::expect_true(any(class(bool_trie) == "trie"))
})

testthat::context("Test conversion of tries into other R objects")

testthat::test_that("Tries can be turned into lists", {
  trie_inst <- trie("foo", "bar")
  trlist <- as.list(trie_inst)
  testthat::expect_true(is.list(trlist))
  testthat::expect_equal(length(trlist), 2)
  testthat::expect_equal(names(trlist), c("keys", "values"))
  testthat::expect_equal(trlist$values, "bar")
  testthat::expect_equal(trlist$keys, "foo")
})

testthat::test_that("Tries can be turned into lists", {
  trie_inst <- trie("foo", "bar")
  trlist <- as.data.frame(trie_inst)
  testthat::expect_true(is.data.frame(trlist))
  testthat::expect_equal(ncol(trlist), 2)
  testthat::expect_equal(names(trlist), c("keys", "values"))
  testthat::expect_equal(trlist$values, "bar")
  testthat::expect_equal(trlist$keys, "foo")
})

#'@title Add or remove trie entries
#'
#'@description \code{trie_add} and \code{trie_remove} allow you to
#'add or remove entries from tries, respectively.
#'
#'@param trie a trie object created with \code{\link{trie}}
#'
#'@param keys a character vector containing the keys of the entries to
#'add (or remove). Entries with NA keys will not be added.
#'
#'@param values an atomic vector, matching the type of the trie, containing
#'the values of the entries to add. Entries with NA values will not be added.
#'
#'@return nothing; the trie is modified in-place
#'
#'@examples
#'trie <- trie("foo", "bar")
#'length(trie)
#'
#'trie_add(trie, "baz", "qux")
#'length(trie)
#'
#'trie_remove(trie, "baz")
#'length(trie)
#'
#'@seealso \code{\link{trie}} for creating tries in the first place.
#'@name alter
#'@rdname alter
#'@export
trie_add <- function(trie, keys, values){
  stopifnot(length(keys) == length(values))
  stopifnot(is.character(keys))
  UseMethod("trie_add", trie)
}

#'@export
trie_add.string_trie <- function(trie, keys, values){
  stopifnot(is.character(values))
  add_trie_string(trie, keys, values)
  return(invisible())
}

#'@export
trie_add.integer_trie <- function(trie, keys, values){
  stopifnot(is.integer(values))
  add_trie_integer(trie, keys, values)
  return(invisible())
}

#'@export
trie_add.numeric_trie <- function(trie, keys, values){
  stopifnot(is.numeric(values))
  add_trie_numeric(trie, keys, values)
  return(invisible())
}

#'@export
trie_add.logical_trie <- function(trie, keys, values){
  stopifnot(is.logical(values))
  add_trie_logical(trie, keys, values)
  return(invisible())
}

#'@rdname alter
#'@export
trie_remove <- function(trie, keys){
  stopifnot(is.character(keys))
  UseMethod("trie_remove", trie)
}

#'@export
trie_remove.string_trie <- function(trie, keys){
  remove_trie_string(trie, keys)
  return(invisible())
}

#'@export
trie_remove.integer_trie <- function(trie, keys){
  remove_trie_integer(trie, keys)
  return(invisible())
}

#'@export
trie_remove.numeric_trie <- function(trie, keys){
  remove_trie_numeric(trie, keys)
  return(invisible())
}

#'@export
trie_remove.logical_trie <- function(trie, keys){
  remove_trie_logical(trie, keys)
  return(invisible())
}

context("Test trie alteration")

testthat::test_that("String tries can be altered", {
  trie <- trie("foo", "bar")
  original_length <- length(trie)
  trie_add(trie, "baz", "qux")
  increased_length <- length(trie)
  trie_remove(trie, "baz")

  testthat::expect_true(original_length < increased_length)
  testthat::expect_true(length(trie) == original_length)
})

testthat::test_that("String tries can be altered", {
  trie <- trie("foo", "bar")
  original_length <- length(trie)
  trie_add(trie, "baz", "qux")
  increased_length <- length(trie)
  trie_remove(trie, "baz")

  testthat::expect_true(original_length < increased_length)
  testthat::expect_true(length(trie) == original_length)
})

testthat::test_that("Integer tries can be altered", {
  trie <- trie("foo", 1)
  original_length <- length(trie)
  trie_add(trie, "baz", 2)
  increased_length <- length(trie)
  trie_remove(trie, "baz")

  testthat::expect_true(original_length < increased_length)
  testthat::expect_true(length(trie) == original_length)
})


testthat::test_that("Numeric tries can be altered", {
  trie <- trie("foo", as.numeric(1))
  original_length <- length(trie)
  trie_add(trie, "baz", as.numeric(2))
  increased_length <- length(trie)
  trie_remove(trie, "baz")

  testthat::expect_true(original_length < increased_length)
  testthat::expect_true(length(trie) == original_length)
})

testthat::test_that("Logical tries can be altered", {
  trie <- trie("foo", FALSE)
  original_length <- length(trie)
  trie_add(trie, "baz", TRUE)
  increased_length <- length(trie)
  trie_remove(trie, "baz")

  testthat::expect_true(original_length < increased_length)
  testthat::expect_true(length(trie) == original_length)
})

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

add_trie_string <- function(trie, keys, values) {
    invisible(.Call(`_triebeard_add_trie_string`, trie, keys, values))
}

add_trie_integer <- function(trie, keys, values) {
    invisible(.Call(`_triebeard_add_trie_integer`, trie, keys, values))
}

add_trie_numeric <- function(trie, keys, values) {
    invisible(.Call(`_triebeard_add_trie_numeric`, trie, keys, values))
}

add_trie_logical <- function(trie, keys, values) {
    invisible(.Call(`_triebeard_add_trie_logical`, trie, keys, values))
}

remove_trie_string <- function(trie, keys) {
    invisible(.Call(`_triebeard_remove_trie_string`, trie, keys))
}

remove_trie_integer <- function(trie, keys) {
    invisible(.Call(`_triebeard_remove_trie_integer`, trie, keys))
}

remove_trie_numeric <- function(trie, keys) {
    invisible(.Call(`_triebeard_remove_trie_numeric`, trie, keys))
}

remove_trie_logical <- function(trie, keys) {
    invisible(.Call(`_triebeard_remove_trie_logical`, trie, keys))
}

radix_create_string <- function(keys, values) {
    .Call(`_triebeard_radix_create_string`, keys, values)
}

radix_create_integer <- function(keys, values) {
    .Call(`_triebeard_radix_create_integer`, keys, values)
}

radix_create_numeric <- function(keys, values) {
    .Call(`_triebeard_radix_create_numeric`, keys, values)
}

radix_create_logical <- function(keys, values) {
    .Call(`_triebeard_radix_create_logical`, keys, values)
}

get_keys_string <- function(radix) {
    .Call(`_triebeard_get_keys_string`, radix)
}

get_keys_integer <- function(radix) {
    .Call(`_triebeard_get_keys_integer`, radix)
}

get_keys_numeric <- function(radix) {
    .Call(`_triebeard_get_keys_numeric`, radix)
}

get_keys_logical <- function(radix) {
    .Call(`_triebeard_get_keys_logical`, radix)
}

get_values_string <- function(radix) {
    .Call(`_triebeard_get_values_string`, radix)
}

get_values_integer <- function(radix) {
    .Call(`_triebeard_get_values_integer`, radix)
}

get_values_numeric <- function(radix) {
    .Call(`_triebeard_get_values_numeric`, radix)
}

get_values_logical <- function(radix) {
    .Call(`_triebeard_get_values_logical`, radix)
}

greedy_string <- function(radix, to_match, include_keys) {
    .Call(`_triebeard_greedy_string`, radix, to_match, include_keys)
}

greedy_integer <- function(radix, to_match, include_keys) {
    .Call(`_triebeard_greedy_integer`, radix, to_match, include_keys)
}

greedy_numeric <- function(radix, to_match, include_keys) {
    .Call(`_triebeard_greedy_numeric`, radix, to_match, include_keys)
}

greedy_logical <- function(radix, to_match, include_keys) {
    .Call(`_triebeard_greedy_logical`, radix, to_match, include_keys)
}

radix_len_string <- function(radix) {
    .Call(`_triebeard_radix_len_string`, radix)
}

radix_len_integer <- function(radix) {
    .Call(`_triebeard_radix_len_integer`, radix)
}

radix_len_numeric <- function(radix) {
    .Call(`_triebeard_radix_len_numeric`, radix)
}

radix_len_logical <- function(radix) {
    .Call(`_triebeard_radix_len_logical`, radix)
}

longest_string <- function(radix, to_match, include_keys) {
    .Call(`_triebeard_longest_string`, radix, to_match, include_keys)
}

longest_integer <- function(radix, to_match, include_keys) {
    .Call(`_triebeard_longest_integer`, radix, to_match, include_keys)
}

longest_numeric <- function(radix, to_match, include_keys) {
    .Call(`_triebeard_longest_numeric`, radix, to_match, include_keys)
}

longest_logical <- function(radix, to_match, include_keys) {
    .Call(`_triebeard_longest_logical`, radix, to_match, include_keys)
}

prefix_string <- function(radix, to_match, include_keys) {
    .Call(`_triebeard_prefix_string`, radix, to_match, include_keys)
}

prefix_integer <- function(radix, to_match, include_keys) {
    .Call(`_triebeard_prefix_integer`, radix, to_match, include_keys)
}

prefix_numeric <- function(radix, to_match, include_keys) {
    .Call(`_triebeard_prefix_numeric`, radix, to_match, include_keys)
}

prefix_logical <- function(radix, to_match, include_keys) {
    .Call(`_triebeard_prefix_logical`, radix, to_match, include_keys)
}

trie_str_string <- function(radix) {
    invisible(.Call(`_triebeard_trie_str_string`, radix))
}

trie_str_integer <- function(radix) {
    invisible(.Call(`_triebeard_trie_str_integer`, radix))
}

trie_str_numeric <- function(radix) {
    invisible(.Call(`_triebeard_trie_str_numeric`, radix))
}

trie_str_logical <- function(radix) {
    invisible(.Call(`_triebeard_trie_str_logical`, radix))
}

#'@export
length.string_trie <- function(x){
  return(radix_len_string(x))
}

#'@export
length.integer_trie <- function(x){
  return(radix_len_integer(x))
}

#'@export
length.numeric_trie <- function(x){
  return(radix_len_numeric(x))
}

#'@export
length.logical_trie <- function(x){
  return(radix_len_logical(x))
}

#'@export
dim.trie <- function(x){
  return(length(x))
}

#'@export
str.trie <- function(object, ...){
  type <- class(object)[3]
  cat(paste0(type, "\n"))

  switch(type,
         "string_trie"  = {trie_str_string(object)},
         "integer_trie" = {trie_str_integer(object)},
         "numeric_trie" = {trie_str_numeric(object)},
         "logical_trie" = {trie_str_logical(object)}
         )

  return(invisible())
}

#'@export
print.trie <- function(x, ...){
  len <- length(x)
  entry_word <- ifelse(len != 1, "entries", "entry")
  cat("A", class(x)[3], "object with", len, entry_word, "\n")
}

---
title: "Radix trees in Rcpp"
author: "Oliver Keyes"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Radix trees in Rcpp}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

A **radix tree** is a data structure optimised for storing key-value pairs in a way optimised for searching. This makes them very, very good for efficiently matching data against keys, and retrieving the values *associated* with those keys.

`triebeard` provides an implementation of radix trees for Rcpp (and also for use directly in R). To start using radix trees in your Rcpp development, simply modify your C++ file to include at the top:

```{Rcpp, eval=FALSE}
//[[Rcpp::depends(triebeard)]]
#include <radix.h>
```

## Constructing trees

Trees are constructed using the syntax:

```{Rcpp, eval=FALSE}
radix_tree<type1, type2> radix;
```

Where `type` represents the type of the keys (for example, `std::string`) and `type2` the type of the values.

Radix trees can have any scalar type as keys, although strings are most typical; they can also have any scalar type for values. Once you've constructed a tree, new entries can be added in a very R-like way: `radix[new_key] = new_value;`. Entries can also be removed, with `radix.erase(key)`.

## Matching against trees

We then move on to the fun bit: matching! As mentioned, radix trees are really good for matching arbitrary values against keys (well, keys of the same type) and retrieving the associated values.

There are three types of supported matching; longest, prefix, and greedy. Longest does exactly what it says on the tin: it finds the key-value pair where the longest initial part of the key matches the arbitrary value:

```{Rcpp, eval=FALSE}
radix_tree<std::string, std::string> radix;
radix["turnin"] = "entry the first";
radix["turin"] = "entry the second";

radix_tree<std::string, std::string>::iterator it;

it = radix.longest_match("turing");

if(it = radix.end()){
  printf("No match was found :(");
} else {
  std::string result = "Key of longest match: " + it->first + " , value of longest match: " + it->second;
}
```

Prefix matching provides all trie entries where the value-to-match is a *prefix* of the key:

```{Rcpp, eval=FALSE}
radix_tree<std::string, std::string> radix;
radix["turnin"] = "entry the first";
radix["turin"] = "entry the second";

std::vector<radix_tree<std::string, std::string>::iterator> vec;
std::vector<radix_tree<std::string, std::string>::iterator>::iterator it;

it = radix.prefix_match("tur");

if(it == vec.end()){
  printf("No match was found :(");
} else {
  for (it = vec.begin(); it != vec.end(); ++it) {
    std::string result = "Key of a prefix match: " + it->first + " , value of a prefix match: " + it->second;
  }
}
```

Greedy matching matches very, very fuzzily (a value of 'bring', for example, will match 'blind', 'bind' and 'binary') and, syntactically, looks exactly the same as prefix-matching, albeit with `radix.greedy_match()` instead of `radix.prefix_match()`.


### Other trie things

If you have ideas for other trie-like structures, or functions that would be useful with *these* tries, the best approach
is to either [request it](https://github.com/Ironholds/triebeard/issues) or [add it](https://github.com/Ironholds/triebeard/pulls)!

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/match.R
\name{longest_match}
\alias{longest_match}
\title{Find the longest match in a trie}
\usage{
longest_match(trie, to_match)
}
\arguments{
\item{trie}{a trie object, created with \code{\link{trie}}}

\item{to_match}{a character vector containing the strings to match against the
trie's keys.}
}
\description{
\code{longest_match} accepts a trie and a character vector
and returns the value associated with whichever key had the \emph{longest match}
to each entry in the character vector. A trie of "binary" and "bind", for example,
with an entry-to-compare of "binder", will match to "bind".
}
\examples{
trie <- trie(keys = c("afford", "affair", "available", "binary", "bind", "blind"),
                   values = c("afford", "affair", "available", "binary", "bind", "blind"))
longest_match(trie, "binder")

}
\seealso{
\code{\link{prefix_match}} and \code{\link{greedy_match}}
for prefix and greedy matching, respectively.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/match.R
\name{greedy_match}
\alias{greedy_match}
\title{Greedily match against a tree}
\usage{
greedy_match(trie, to_match)
}
\arguments{
\item{trie}{a trie object, created with \code{\link{trie}}}

\item{to_match}{a character vector containing the strings to check against the
trie's keys.}
}
\value{
a list, the length of \code{to_match}, with each entry containing any trie values
where the \code{to_match} element greedily matches the associated key. In the case that
nothing was found, the entry will contain \code{NA}.
}
\description{
\code{greedy_match} accepts a trie and a character vector
and returns the values associated with any key that is "greedily" (read: fuzzily)
matched against one of the character vector entries.
}
\examples{
trie <- trie(keys = c("afford", "affair", "available", "binary", "bind", "blind"),
                   values = c("afford", "affair", "available", "binary", "bind", "blind"))
greedy_match(trie, c("avoid", "bring", "attack"))

}
\seealso{
\code{\link{longest_match}} and \code{\link{prefix_match}}
for longest and prefix matching, respectively.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/match.R
\name{prefix_match}
\alias{prefix_match}
\title{Find the prefix matches in a trie}
\usage{
prefix_match(trie, to_match)
}
\arguments{
\item{trie}{a trie object, created with \code{\link{trie}}}

\item{to_match}{a character vector containing the strings to check against the
trie's keys.}
}
\value{
a list, the length of \code{to_match}, with each entry containing any trie values
where the \code{to_match} element was a prefix of the associated key. In the case that
nothing was found, the entry will contain \code{NA}.
}
\description{
\code{prefix_match} accepts a trie and a character vector
and returns the values associated with any key that has a particular
character vector entry as a prefix (see the examples).
}
\examples{
trie <- trie(keys = c("afford", "affair", "available", "binary", "bind", "blind"),
                   values = c("afford", "affair", "available", "binary", "bind", "blind"))
prefix_match(trie, "aff")

}
\seealso{
\code{\link{longest_match}} and \code{\link{greedy_match}}
for longest and greedy matching, respectively.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get.R
\name{getters}
\alias{get_keys}
\alias{get_values}
\alias{getters}
\title{Trie Getters}
\usage{
get_keys(trie)

get_values(trie)
}
\arguments{
\item{trie}{A trie object, created with \code{\link{trie}}.}
}
\value{
An atomic vector of keys or values stored in the trie.
}
\description{
"Getters" for the data stored in a trie object. \code{get_keys}
gets the keys, \code{get_values} gets the values.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create.R
\name{trie}
\alias{trie}
\title{Create a Trie}
\usage{
trie(keys, values)
}
\arguments{
\item{keys}{a character vector containing the keys for the trie.}

\item{values}{an atomic vector of any type, containing the values to pair with
\code{keys}. Must be the same length as \code{keys}.}
}
\value{
a `trie` object.
}
\description{
\code{create_trie} creates a trie (a key-value store optimised
for matching) out of a provided character vector of keys, and a numeric,
character, logical or integer vector of values (both the same length).
}
\examples{
# An integer trie
int_trie <- trie(keys = "foo", values = 1)

# A string trie
str_trie <- trie(keys = "foo", values = "bar")

}
\seealso{
\code{\link{trie_add}} and \code{\link{trie_remove}} for adding to and removing
from tries after their creation, and \code{\link{longest_match}} and other match functions
for matching values against the keys of a created trie.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/triebeard.R
\docType{package}
\name{triebeard}
\alias{triebeard}
\alias{triebeard-package}
\title{Radix trees in Rcpp}
\description{
This package provides access to Radix tree (or "trie") structures in Rcpp. At
a later date it will hopefully provide them in R, too.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/alter.R
\name{alter}
\alias{alter}
\alias{trie_add}
\alias{trie_remove}
\title{Add or remove trie entries}
\usage{
trie_add(trie, keys, values)

trie_remove(trie, keys)
}
\arguments{
\item{trie}{a trie object created with \code{\link{trie}}}

\item{keys}{a character vector containing the keys of the entries to
add (or remove). Entries with NA keys will not be added.}

\item{values}{an atomic vector, matching the type of the trie, containing
the values of the entries to add. Entries with NA values will not be added.}
}
\value{
nothing; the trie is modified in-place
}
\description{
\code{trie_add} and \code{trie_remove} allow you to
add or remove entries from tries, respectively.
}
\examples{
trie <- trie("foo", "bar")
length(trie)

trie_add(trie, "baz", "qux")
length(trie)

trie_remove(trie, "baz")
length(trie)

}
\seealso{
\code{\link{trie}} for creating tries in the first place.
}
